.packageName <- "makecdfenv"
## Laurent 2002 tneruaL
## Class definitions for 'Cdf'
setClass("Cdf", representation(cdfName="character",
                                 name="matrix",
                                 name.levels="character",
                                 pbase="matrix",
                                 pbase.levels="character",
                                 tbase="matrix",
                                 tbase.levels="character",
                                 atom="matrix"))
  ## name.levels method
  if( !isGeneric("name.levels") ) {
    setGeneric("name.levels", function(object)
               standardGeneric("name.levels"))
  } else
    cat("name.levels is already generic, could be a problem.\n")

   setMethod("name.levels","Cdf",function(object) object@name.levels)

  if( !isGeneric("name.levels<-") )
      setGeneric("name.levels<-", function(object, value)
                 standardGeneric("name.levels<-"))
  setReplaceMethod("name.levels", "Cdf", function(object, value){
    object@name.levels <- value
    object
    })

  ## pbase method
  if( !isGeneric("pbase") ) {
    setGeneric("pbase", function(object)
               standardGeneric("pbase"))
  } else
    cat("pbase is already generic, could be a problem.\n")

   setMethod("pbase","Cdf",function(object) object@pbase)

  if( !isGeneric("pbase<-") )
      setGeneric("pbase<-", function(object, value)
                 standardGeneric("pbase<-"))
  setReplaceMethod("pbase", "Cdf", function(object, value){
    object@pbase <- value
    object
    })

 ##pbase.levels method
  if( !isGeneric("pbase.levels") ) {
    setGeneric("pbase.levels", function(object)
               standardGeneric("pbase.levels"))
  } else
    cat("pbase.levels is already generic, could be a problem.\n")

   setMethod("pbase.levels","Cdf",function(object) object@pbase.levels)

  if( !isGeneric("pbase.levels<-") )
      setGeneric("pbase.levels<-", function(object, value)
                 standardGeneric("pbase.levels<-"))
  setReplaceMethod("pbase.levels", "Cdf", function(object, value){
    object@pbase.levels <- value
    object
    })

  ## tbase method
  if( !isGeneric("tbase") ) {
    setGeneric("tbase", function(object)
               standardGeneric("tbase"))
  } else
    cat("tbase is already generic, could be a problem.\n")

   setMethod("tbase","Cdf",function(object) object@tbase)

  if( !isGeneric("tbase<-") )
      setGeneric("tbase<-", function(object, value)
                 standardGeneric("tbase<-"))
  setReplaceMethod("tbase", "Cdf", function(object, value){
    object@tbase <- value
    object
    })

  ## tbase.levels method
  if( !isGeneric("tbase.levels") ) {
    setGeneric("tbase.levels", function(object)
               standardGeneric("tbase.levels"))
  } else
    cat("tbase.levels is already generic, could be a problem.\n")

   setMethod("tbase.levels","Cdf",function(object) object@tbase.levels)

  if( !isGeneric("tbase.levels<-") )
      setGeneric("tbase.levels<-", function(object, value)
                 standardGeneric("tbase.levels<-"))
  setReplaceMethod("tbase.levels", "Cdf", function(object, value){
    object@tbase.levels <- value
    object
    })

  ## printing method for 'Cdf'
  setMethod("show", "Cdf",
            function(object) {
              ##print("Cdf object\n")
              cat("Cdf object:\n")
              cat("cdfName=", object@cdfName)
              cat("dim(name)=",dim(object@name)[1],"x",
                  dim(object@name)[2]," features\n",sep="")
              cat("nb. affyid=", length(object@name.levels), "\n", sep="")
            })



getInfoInFile <- function(filename, type, unit, property, compress=NULL) {
  if (is.null(compress) && (type == "CDF"))
    compress <- getOption("BioC")$affy$compress.cdf
  if (is.null(compress) && (type == "CEL"))
    compress <- getOption("BioC")$affy$compress.cel
  .Call("getInfo", as.character(filename), as.character(type), as.character(unit), as.character(property), as.integer(compress))
}
##------------------------------------------------------------
## (C) Rafael Irizarry, Wolfgang Huber 2003
##------------------------------------------------------------
make.cdf.env <- function(filename,
                         cdf.path = getwd(),
                         compress = FALSE,
                         return.env.only = TRUE,
                         verbose = TRUE) {
  stopifnot(is.logical(verbose), is.logical(return.env.only), is.logical(compress),
            is.character(cdf.path), is.character(filename))
  stopifnot(length(filename)==1, length(cdf.path)==1, length(return.env.only)==1,
            length(verbose)==1, length(compress)==1)
  
  ## read in the cdf file into a CDF object
  if(verbose)
    cat("Reading CDF file.\n")
  cdf <- read.cdffile(file.path(path.expand(cdf.path),filename), compress=compress)

  if(verbose)
    cat("Creating CDF environment\n")
  
  ## extract number or rows and number of columns
  sizex <- dim(cdf@name)[1]
  sizey <- dim(cdf@name)[2]
  
  ## where are the pm and mm
  pm.or.mm <- as.vector(pmormm(cdf))
  pmindex <- pm.or.mm
  mmindex <- !pm.or.mm
  rm(pm.or.mm)
  
  ## get ready to go
  ## start the new environment
  ## position in vector will correspond to the position on the array
  ## given ncol and nrow we can figure out the 2D (x,y) position
  ## now we assign 

  env = new.env(hash=TRUE, parent=NULL)
  
  genenames <- name.levels(cdf) ##so that we only use name.level method once
  n <- length(genenames) ##number of genes
  probenames <-  as.vector(cdf@name)
  position <- 1:length(probenames)
  numbers <- as.vector(cdf@atom)##number relates to position in genome
  tmp<- split(c(position,  pmindex,   mmindex,   numbers),
              c(probenames,probenames,probenames,probenames))
  rm(pmindex,mmindex,position,numbers,probenames)
  names(tmp) <- genenames[as.numeric(names(tmp))]

  ## this list will be converted to an environment using multiassign
  if(verbose)
    cat("Wait for about",round(length(tmp)/100),"dots")
  tmp <- lapply(tmp,function(x){
    if(verbose)
      if(runif(1)<0.01) cat(".")
    n <- length(x)/4
    position <- x[1:n] 
    pmindex <- which(x[(n+1):(2*n)]==1)
    mmindex <- which(x[(2*n+1):(3*n)]==1)
    numbers <- x[(3*n+1):(4*n)]

    ##get pm and mm and put in order according to numbers
    pm <- position[pmindex][order(numbers[pmindex])]
    mm <- position[mmindex][order(numbers[mmindex])]
    if(length(mm)==0)
      return(cbind(pm=pm,mm=rep(NA,length(pm))))
    else
      return(cbind(pm=pm,mm=mm))
  })
  
  if(verbose)
    cat("\n")
  
  multiassign(names(tmp),tmp, env)

  syms = list(
    I2XY  = paste("y*", sizex, "+x+1", sep=""),
    XY2I  = paste("r=cbind((i-1)%%",sizex,",(i-1)%/%",sizex,"); colnames(r)=c('x','y'); return(r)", sep=""),
    SIZEX = paste(sizex),
    SIZEY = paste(sizey),
    SIZEI = paste(sizex*sizey))

  ## return:
  if (return.env.only) {
    rv = env
  } else {
    rv = list(env=env, syms=syms)
  }
  return(rv)
}

##------------------------------------------------------------
## package maker
##------------------------------------------------------------
make.cdf.package<- function(filename,
                            packagename = NULL,
                            cdf.path = getwd(),
                            package.path = getwd(),
                            compress = FALSE,
                            author = "The Bioconductor Project",
                            maintainer = "The Bioconductor Project <bioconductor@stat.math.ethz.ch>",
                            version = library(help=makecdfenv)$info[[2]][[2]][2],
                            unlink  = FALSE,
                            verbose = TRUE) {
  
  ## If no packagename given change CDF filename to packagename.
  ## cleancdfname() is defined in package 'affy'.
  if(is.null(packagename))
    packagename <- cleancdfname(sub("\.cdf$", "", filename, ignore.case=TRUE))

  cdf <- make.cdf.env(filename, cdf.path=cdf.path, compress=compress, return.env.only=FALSE)
  assign(packagename, cdf$env)

  home  <- .path.package("makecdfenv")
  symbols = append(cdf$syms,
    list(PKGNAME     = packagename,
         VERSION     = version,
         AUTHOR      = author,
         TODAY       = date(),
         CDFFILENAME = filename,
         MAINTAINER  = maintainer))

  ## see in the subdirectory 'Code' of this package for all the
  ## files that go into the package!
  res = createPackage(packagename,
                destinationDir = path.expand(package.path),
                originDir  = file.path(home, "Code"),
                symbolValues = symbols,
                unlink = unlink, quiet = !verbose)

  ## save an XDR file with the environment
  save(list = packagename, file = file.path(res$pkgdir, "data", paste(packagename,".rda",sep="")))
  
  return(packagename)
}








# originally called 'getnature' in affyR

pmormm <- function(cdf) {
  a.i <- which(cdf@pbase.levels == "A")
  t.i <- which(cdf@pbase.levels == "T")
  g.i <- which(cdf@pbase.levels == "G")
  c.i <- which(cdf@pbase.levels == "C")
  md <- dim(cdf@name)

  # init to 'NA'
  nature <- matrix(NA,md[1],md[2])
  i <- which(
             ((cdf@pbase == a.i) & (cdf@tbase == t.i)) | 
             ((cdf@pbase == t.i) & (cdf@tbase == a.i)) |
             ((cdf@pbase == g.i) & (cdf@tbase == c.i)) |
             ((cdf@pbase == c.i) & (cdf@tbase == g.i))
             )
  # set the PM to TRUE
  nature[i] <- TRUE
  i <- which((! is.na(cdf@atom)) & is.na(nature))
  # set the MM to FALSE
  nature[i] <- FALSE
  return(nature)   
}


















read.cdffile <- function(file, compress=FALSE) {
  ff <- new("Cdf")

  tmp <- getInfoInFile(file, "CDF", unit="Chip", property="Name", compress=compress)
  tmp <- substr(tmp, 1, nchar(tmp)-2)

  ff@cdfName <- tmp

  tmp <- .Call("readCDFfile", as.character(file), as.integer(3), as.integer(compress))
  tmp[tmp == ""] <- NA
  mydim <- dim(tmp)

  tmp <- factor(tmp)
  
  ff@name <- array(as.integer(tmp), mydim)

  ff@name.levels <- levels(tmp)
  rm(tmp)
  gc()
  tmp <- .Call("readCDFfile", as.character(file),
               as.integer(7), as.integer(compress))
  tmp[tmp == ""] <- NA
  mydim <- dim(tmp)
  
  tmp <- factor(tmp)
  
  ff@pbase <- array(as.integer(tmp), mydim)

  ff@pbase.levels <- levels(tmp)
  rm(tmp)
  gc()
  tmp <- .Call("readCDFfile", as.character(file),
               as.integer(8), as.integer(compress))
  tmp[tmp == ""] <- NA
  mydim <- dim(tmp)
  tmp <- factor(tmp)

  ff@tbase <- array(as.integer(tmp), mydim)

  ff@tbase.levels <- levels(tmp)
  rm(tmp)
  gc()
  
  tmp <- .Call("readCDFfile", as.character(file),
               as.integer(9), as.integer(compress))
  tmp[tmp == ""] <- NA
  mydim <- dim(tmp)

  ff@atom <- array(as.integer(tmp), mydim)

  gc() 

  return(ff)
}

##-----------------------------------------------------------------
## .First.lib: this function is called when the package is loaded
##-----------------------------------------------------------------
.First.lib <- function(libname, pkgname, where) {
  library.dynam("makecdfenv", pkgname, libname)
}

